@@ -479,6 +479,16 @@ vasnprintf (char *resultbuf, size_t *lengthp, const char *format, va_list args)
 			  + 1 /* turn floor into ceil */
 			  + 2; /* account for leading sign or alternate form */
 		      else
+# endif
+# ifdef HAVE_INT64_AND_I64
+		      if (type == TYPE_INT64 || type == TYPE_UINT64)
+			tmp_length =
+			  (unsigned int) (sizeof (unsigned __int64) * CHAR_BIT
+					  * 0.25 /* binary -> hexadecimal */
+					  )
+			  + 1 /* turn floor into ceil */
+			  + 2; /* account for leading sign or alternate form */
+		      else
 # endif
 		      if (type == TYPE_LONGINT || type == TYPE_ULONGINT)
 			tmp_length =
@@ -853,11 +863,19 @@ vasnprintf (char *resultbuf, size_t *lengthp, const char *format, va_list args)
 				}
 			    }
 			  
-			  count = print_long_long (result + length, maxlen,
+#if HAVE_SNPRINTF
+ 			  count = print_long_long (result + length, maxlen,
+ 						   width, precision,
+ 						   dp->flags,
+ 						   dp->conversion,
+ 						   arg);
+#else
+			  count = print_long_long (tmp, tmp_length,
 						   width, precision,
 						   dp->flags,
 						   dp->conversion,
 						   arg);
+#endif
 			}
 			break;
 #else
